# MazeExplorer

MazeExplorer is a customisable 3D benchmark for assessing generalisation in Reinforcement Learning.

It is based on the 3D first-person game [Doom](https://en.wikipedia.org/wiki/Doom_(1993_video_game)) and the open-source
environment [VizDoom](https://github.com/mwydmuch/ViZDoom).

This repository contains the code for the MazeExplorer Gym Environment along with the scripts to generate baseline results.

By Luke Harries*, Sebastian Lee*, Jaroslaw Rzepecki, Katja Hofmann, and Sam Devlin.  
\* Joint first author

![Default textures](https://github.com/microsoft/MazeExplorer/raw/master/assets/default_textures.png) ![Random Textures](https://github.com/microsoft/MazeExplorer/raw/master/assets/textures-1.png) ![Random Textures](https://github.com/microsoft/MazeExplorer/raw/master/assets/textures-2.png) 

# The Mission

The goal is to navigate a procedurally generated maze and collect a set number of keys.

The environment is highly customisable, allowing you to create different training and test environments.

The following features of the environment can be configured:
- Number of maps
- Map Size (X, Y)
- Maze complexity
- Maze density
- Random/Fixed keys
- Random/Fixed textures
- Random/Fixed spawn
- Number of keys
- Environment Seed
- Episode timeout
- Reward clipping
- Frame stack
- Resolution
- Action frame repeat
- Actions space
- Specific textures (Wall,
ceiling, floor)

# Example Usage

```python
from mazeexplorer import MazeExplorer

train_env = MazeExplorer(number_maps=1,
              size=(15, 15),
              random_spawn=True,
              random_textures=False,
              keys=6)
              
test_env = MazeExplorer(number_maps=1,
              size=(15, 15),
              random_spawn=True,
              random_textures=False,
              keys=6)

# training
for _ in range(1000):
    obs, rewards, dones, info = train_env.step(train_env.action_space.sample())
    
    
# testing
for _ in range(1000):
    obs, rewards, dones, info = test_env.step(test_env.action_space.sample())
```

# Installation

1. Install the dependencies for VizDoom: [Linux](https://github.com/mwydmuch/ViZDoom/blob/master/doc/Building.md#-linux), [MacOS](https://github.com/mwydmuch/ViZDoom/blob/master/doc/Building.md#-linux) or [Windows](https://github.com/mwydmuch/ViZDoom/blob/master/doc/Building.md#-windows).
1. `pip3 install virtualenv pytest`
1. Create a virtualenv and activate it
    1. `virtualenv mazeexplorer-env`
    1. `source maze-env/bin/activate`
1. Git clone this repo `git clone https://github.com/microsoft/MazeExplorer`
1. cd into the repo: `cd MazeExplorer`
1. Pull the submodules with `git submodule update --init --recursive`
1. Install the dependencies: `pip3 install -e .`
1. Run the tests: `bash test.sh`

# Baseline experiments

The information to reproduce the baseline experiments is shown in `baseline_experiments/experiments.md`.

# Contributing

This project welcomes contributions and suggestions.  Most contributions require you to agree to a
Contributor License Agreement (CLA) declaring that you have the right to, and actually do, grant us
the rights to use your contribution. For details, visit https://cla.microsoft.com.

When you submit a pull request, a CLA-bot will automatically determine whether you need to provide
a CLA and decorate the PR appropriately (e.g., label, comment). Simply follow the instructions
provided by the bot. You will only need to do this once across all repos using our CLA.

This project has adopted the [Microsoft Open Source Code of Conduct](https://opensource.microsoft.com/codeofconduct/).
For more information see the [Code of Conduct FAQ](https://opensource.microsoft.com/codeofconduct/faq/) or
contact [opencode@microsoft.com](mailto:opencode@microsoft.com) with any additional questions or comments.
